==> eg_particle_interactions-v0.8-fabric-mc1.21.8-sources.jar: games/enchanted/eg_particle_interactions/common/mixin/particles/FireflyParticleMixin.java <==
package games.enchanted.eg_particle_interactions.common.mixin.particles;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import games.enchanted.eg_particle_interactions.common.config.ConfigHandler;
import games.enchanted.eg_particle_interactions.common.duck.ParticleAccess;
import games.enchanted.eg_particle_interactions.common.particle_override.BlockParticleOverrides;
import games.enchanted.eg_particle_interactions.common.registry.BlockOrTagLocation;
import games.enchanted.eg_particle_interactions.common.registry.RegistryHelpers;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.List;
import java.util.Objects;
import java.util.function.Supplier;
import net.minecraft.class_10748;
import net.minecraft.class_2246;
import net.minecraft.class_2680;
import net.minecraft.class_4003;
import net.minecraft.class_638;

@Mixin(class_10748.class)
public abstract class FireflyParticleMixin extends class_4003 {
    protected FireflyParticleMixin(class_638 level, double x, double y, double z) {
        super(level, x, y, z);
    }

    @Inject(
        at = @At("TAIL"),
        method = "<init>"
    )
    private void block_place_particle$makeFirefliesNotGetStuckOnStuff(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, CallbackInfo ci) {
        if(!ConfigHandler.general_fireflyFixes) return;
        ((ParticleAccess) this).eg_particle_interactions$setBypassMovementCollisionCheck(true);
    }

    @WrapOperation(
        at = @At(value = "INVOKE", target = "Lnet/minecraft/world/level/block/state/BlockState;isAir()Z"),
        method = "tick"
    )
    public boolean block_place_particle$makeFirefliesNotDieInFireflyBushes(class_2680 state, Operation<Boolean> original) {
        if(!ConfigHandler.general_fireflyFixes) {
            return original.call(state);
        }
        Supplier<List<BlockOrTagLocation>> fireflyOverrideBlocks = BlockParticleOverrides.FIREFLY.getSupportedBlockResourceLocations_getter();
        Supplier<List<BlockOrTagLocation>> grassBladeOverrideBlocks = BlockParticleOverrides.GRASS_BLADE.getSupportedBlockResourceLocations_getter();
        if(fireflyOverrideBlocks == null || grassBladeOverrideBlocks == null) {
            return original.call(state);
        }
        if(
            fireflyOverrideBlocks.get().contains(new BlockOrTagLocation(RegistryHelpers.getLocationFromBlock(state.method_26204()), false)) ||
            grassBladeOverrideBlocks.get().contains(new BlockOrTagLocation(RegistryHelpers.getLocationFromBlock(state.method_26204()), false)))
        {
            return original.call(class_2246.field_10124.method_9564());
        }
        return original.call(state);
    }
}

==> eg_particle_interactions-v0.8.3-fabric-mc1.21.10-sources.jar: games/enchanted/eg_particle_interactions/common/mixin/particles/FireflyParticleMixin.java <==
package games.enchanted.eg_particle_interactions.common.mixin.particles;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import games.enchanted.eg_particle_interactions.common.config.ConfigHandler;
import games.enchanted.eg_particle_interactions.common.duck.ParticleAccess;
import games.enchanted.eg_particle_interactions.common.particle_override.BlockParticleOverrides;
import games.enchanted.eg_particle_interactions.common.registry.BlockOrTagLocation;
import games.enchanted.eg_particle_interactions.common.registry.RegistryHelpers;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import java.util.List;
import java.util.function.Supplier;
import net.minecraft.class_1058;
import net.minecraft.class_10748;
import net.minecraft.class_2246;
import net.minecraft.class_2680;
import net.minecraft.class_3940;
import net.minecraft.class_638;

@Mixin(class_10748.class)
public abstract class FireflyParticleMixin
    //? if minecraft: <= 1.21.8 {
    /*extends TextureSheetParticle
     *///?} else {
    extends class_3940
//?}
{
    //? if minecraft: <=1.21.8 {
    /*protected FireflyParticleMixin(ClientLevel level, double x, double y, double z) {
        super(level, x, y, z);
    }
    *///?} else {
    protected FireflyParticleMixin(class_638 level, double x, double y, double z, class_1058 sprite) {
        super(level, x, y, z, sprite);
    }
    //?}

    //? if minecraft: <=1.21.8 {
    /*@Inject(
        at = @At("TAIL"),
        method = "<init>"
    )
    private void block_place_particle$makeFirefliesNotGetStuckOnStuff(ClientLevel level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, CallbackInfo ci) {
    *///?} else {
    @Inject(
        at = @At("TAIL"),
        method = "<init>"
    )
    private void block_place_particle$makeFirefliesNotGetStuckOnStuff(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_1058 textureAtlasSprite, CallbackInfo ci) {
    //?}
        if(!ConfigHandler.general_fireflyFixes) return;
        ((ParticleAccess) this).eg_particle_interactions$setBypassMovementCollisionCheck(true);
    }

    @WrapOperation(
        at = @At(value = "INVOKE", target = "Lnet/minecraft/world/level/block/state/BlockState;isAir()Z"),
        method = "tick"
    )
    public boolean block_place_particle$makeFirefliesNotDieInFireflyBushes(class_2680 state, Operation<Boolean> original) {
        if(!ConfigHandler.general_fireflyFixes) {
            return original.call(state);
        }
        Supplier<List<BlockOrTagLocation>> fireflyOverrideBlocks = BlockParticleOverrides.FIREFLY.getSupportedBlockResourceLocations_getter();
        Supplier<List<BlockOrTagLocation>> grassBladeOverrideBlocks = BlockParticleOverrides.GRASS_BLADE.getSupportedBlockResourceLocations_getter();
        if(fireflyOverrideBlocks == null || grassBladeOverrideBlocks == null) {
            return original.call(state);
        }
        if(
            fireflyOverrideBlocks.get().contains(new BlockOrTagLocation(RegistryHelpers.getLocationFromBlock(state.method_26204()), false)) ||
            grassBladeOverrideBlocks.get().contains(new BlockOrTagLocation(RegistryHelpers.getLocationFromBlock(state.method_26204()), false)))
        {
            return original.call(class_2246.field_10124.method_9564());
        }
        return original.call(state);
    }
}
